#!/usr/bin/env python

# it2babyk.py, version 0.1

from sys import stderr, version_info
from pytrax import impulsetracker
import json


def die(msg):
    if isinstance(msg, BaseException):
        msg = str(msg)
    stderr.write(str(msg) + '\n')
    exit(1)


if version_info.major < 3:
    die('python3 only!')

MODULE = "test.it"

NOTES = {
    "C-4": 131,
    "C#4": 140,
    "D-4": 145,
    "D#4": 151,
    "E-4": 158,
    "F-4": 161,
    "F#4": 166,
    "G-4": 173,
    "G#4": 178,
    "A-4": 181,
    "A#4": 185,
    "B-4": 189,
    "C-5": 192,
    "C#5": 197,
    "D-5": 200,
    "D#5": 203,
    "E-5": 206,
    "F-5": 208,
    "F#5": 211,
    "G-5": 214,
    "G#5": 216,
    "A-5": 218,
    "A#5": 220,
    "B-5": 222,
    "C-6": 224,
    "C#6": 226,
    "D-6": 227,
    "D#6": 229,
    "E-6": 231,
    "F-6": 233,
    "F#6": 234,
    "G-6": 236,
    "G#6": 237,
    "A-6": 238,
    "A#6": 240,
    "B-6": 241,
    "C-7": 245,
    "C#7": 250,
    "^^^": 1
}

PATTERN_SET = []

SEQUENCE_SET = []


def check_pattern(pattern_array):
    for i, pattern in enumerate(PATTERN_SET):
        if pattern == pattern_array:
            return i
    PATTERN_SET.append(pattern_array)
    return len(PATTERN_SET) - 1


def convert(module, filename):
    output = {
        "title":        "Very Rare VERY!!",
        "artist":       "kleeder         ",
        "copy_info":    "baby-k entry lol",
        "speed_table":  [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
        "volume_table": [10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10],
        "tuning":       2
    }

    for pattern_number in module["orders"]:
        if pattern_number == 254:
            pass
        elif pattern_number != 255:
            pattern0_array = []
            pattern1_array = []
            pattern2_array = []
            pattern3_array = []
            pattern_data = module["patterns"][pattern_number][0]
            for row in pattern_data:
                ch0_set = False
                ch1_set = False
                ch2_set = False
                ch3_set = False
                channel_amount = len(row)
                if channel_amount == 0:
                    pattern0_array.append(0)
                    pattern1_array.append(0)
                    pattern2_array.append(0)
                    pattern3_array.append(0)
                    ch0_set = True
                    ch1_set = True
                    ch2_set = True
                    ch3_set = True
                if channel_amount > 0:
                    for channel in row:
                        if channel['channel'] == 0:
                            ch0_set = True
                            pattern0_array.append(NOTES[channel["notestr"]])
                        elif channel['channel'] == 1:
                            ch1_set = True
                            pattern1_array.append(NOTES[channel["notestr"]])
                        elif channel['channel'] == 2:
                            ch2_set = True
                            pattern2_array.append(NOTES[channel["notestr"]])
                        elif channel['channel'] == 3:
                            ch3_set = True
                            pattern3_array.append(NOTES[channel["notestr"]])
                if not ch0_set:
                    pattern0_array.append(0)
                if not ch1_set:
                    pattern1_array.append(0)
                if not ch2_set:
                    pattern2_array.append(0)
                if not ch3_set:
                    pattern3_array.append(0)
            SEQUENCE_SET.append([
                check_pattern(pattern0_array),
                check_pattern(pattern1_array),
                check_pattern(pattern2_array),
                check_pattern(pattern3_array),
            ])

    while len(PATTERN_SET) < 128:
        PATTERN_SET.append([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])

    while len(SEQUENCE_SET) < 128:
        SEQUENCE_SET.append([255, 255, 255, 255])

    output.update({
        "pattern_order": SEQUENCE_SET,
        "patterns":      PATTERN_SET
    })

    with open(filename, 'w') as fp:
        json.dump(output, fp)


module = impulsetracker.parse_file(MODULE, with_patterns=True)
convert(module, 'output.json')
print("hey i reached the end of the script, the file SHOULD be there now")
